// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"github.com/r3labs/diff/v3"
)

func (s *Settings[C]) notifyHooks(changelog diff.Changelog) {

	if len(changelog) > 0 {
		s.notifyPostprocessingHooks(changelog)
		s.notifyChangeHooks(changelog)
	}

}

func (s *Settings[C]) setConfigInternal(config C) (*Settings[C], diff.Changelog) {

	var (
		changelog diff.Changelog
		err       error
	)

	defer func() {
		s.notifyErrorHooks()
	}()

	if err := validateConfig[C](config); err != nil {
		s.errors = append(s.errors, err)
		return s, changelog
	}

	d, err := diff.NewDiffer()
	if err != nil {
		s.errors = append(s.errors, err)
		return s, changelog
	}

	d.ConvertCompatibleTypes = true
	d.AllowTypeMismatch = true

	changelog, err = d.Diff(s.config, config)
	if err != nil {
		s.errors = append(s.errors, err)
		return s, changelog
	}

	s.config = config
	return s, changelog
}

func (s *Settings[C]) SetConfig(config C) *Settings[C] {
	s.Lock()
	c, l := s.setConfigInternal(config)
	s.Unlock()

	s.notifyHooks(l)

	return c
}
